"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.check = check;

var _getWorkspaceInfo = require("./utils/getWorkspaceInfo");

var _isModularType = require("./utils/isModularType");

var logger = _interopRequireWildcard(require("./utils/logger"));

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

async function check() {
  // ensure that workspaces are setup correctly with yarn§
  // init is a special case where we don't already need to be in a modular repository
  // in this case there's no use checking the workspaces yet because we're setting
  // up a new folder
  const workspace = await (0, _getWorkspaceInfo.getWorkspaceInfo)();

  for (const [packageName, _packageInfo] of Object.entries(workspace)) {
    const packageInfo = _packageInfo;

    if (packageInfo.mismatchedWorkspaceDependencies.length) {
      throw new Error(`${packageName} has mismatchedWorkspaceDependencies ${packageInfo.mismatchedWorkspaceDependencies.join(', ')}`);
    }

    if (!packageInfo.location.startsWith('packages')) {
      throw new Error(`${packageName} is not located within the "/packages" directory in the repository, instead found "${packageInfo.location}"`);
    }

    if (!(0, _isModularType.isValidModularType)(packageInfo.location)) {
      throw new Error(`${packageName} at ${packageInfo.location} is not a valid modular type - Found ${(0, _isModularType.getModularType)(packageInfo.location)}`);
    }

    if (packageInfo.type === 'app') {
      if (packageInfo.public) {
        throw new Error(`${packageName} is marked as "public" - Modular apps should be marked as private.`);
      }
    }

    logger.debug(`${packageName} is valid.`);
  }

  if (process.env.SKIP_PREFLIGHT_CHECK !== 'true') {
    const {
      verifyPackageTree
    } = await Promise.resolve().then(() => _interopRequireWildcard(require('./utils/verifyPackageTree')));
    await verifyPackageTree();
  }
}