"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getYarnWorkspaceInfo = getYarnWorkspaceInfo;
exports.getAllWorkspaces = void 0;

var _execa = _interopRequireDefault(require("execa"));

var _memoize = _interopRequireDefault(require("./memoize"));

var _getModularRoot = _interopRequireDefault(require("./getModularRoot"));

var _stripAnsi = _interopRequireDefault(require("strip-ansi"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getYarnWorkspaceInfo(cwd) {
  // strip out ANSI color codes and escape characters
  const strippedStd = (0, _stripAnsi.default)(_execa.default.sync('yarnpkg', ['--silent', 'workspaces', 'info'], {
    all: true,
    reject: false,
    cwd,
    cleanup: true
  }).stdout // there's an edge case where if there are no packages in the current workspace
  // then this command returns an empty string and no JSON - so we have to default
  // to an empty directory which can be JSON parsed.
  ) || '{}';
  return JSON.parse(strippedStd);
}

function _getAllWorkspaces() {
  const modularRoot = (0, _getModularRoot.default)();
  return getYarnWorkspaceInfo(modularRoot);
}

const getAllWorkspaces = (0, _memoize.default)(_getAllWorkspaces);
exports.getAllWorkspaces = getAllWorkspaces;