"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var path = _interopRequireWildcard(require("path"));

var fs = _interopRequireWildcard(require("fs-extra"));

var _findUp = _interopRequireDefault(require("find-up"));

var _memoize = _interopRequireDefault(require("./memoize"));

var logger = _interopRequireWildcard(require("./logger"));

var _execa = _interopRequireDefault(require("execa"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function isModularRoot(packageJsonPath) {
  var _packageJson$modular;

  const packageJson = fs.readJSONSync(packageJsonPath, {
    encoding: 'utf8'
  });
  return (packageJson === null || packageJson === void 0 ? void 0 : (_packageJson$modular = packageJson.modular) === null || _packageJson$modular === void 0 ? void 0 : _packageJson$modular.type) === 'root';
}

function findUpModularRoot() {
  return _findUp.default.sync(directory => {
    const packageJsonPath = path.join(directory, 'package.json');

    if (_findUp.default.sync.exists(packageJsonPath) && isModularRoot(packageJsonPath)) {
      return packageJsonPath;
    }

    return;
  }, {
    type: 'file',
    allowSymlinks: false
  });
}
/**
 * This is a shortcut - when running in a git repo we bail out and
 * use the git repository root as the modular root. This handles
 * most use cases since people should be tracking their entire repo
 * as a modular repo.
 *
 * @returns string | undefined
 */


function getGitRoot() {
  try {
    const {
      stdout
    } = _execa.default.sync(`git`, [`rev-parse`, `--show-toplevel`]);

    const packageJsonPath = path.join(stdout, 'package.json');

    if (isModularRoot(packageJsonPath)) {
      return stdout;
    }
  } catch (e) {
    /// clearly not in a git repo so just use the CWD
    return undefined;
  }
}

function getModularRoot() {
  try {
    let modularRoot = getGitRoot();

    if (modularRoot) {
      logger.debug(`Using git located modular root.`);
    } else {
      logger.debug('Deferring to find-up to locate modular root');
      modularRoot = findUpModularRoot();

      if (modularRoot === undefined) {
        throw new Error('Could not find modular root.');
      }

      modularRoot = path.dirname(modularRoot);
    }

    logger.debug(`Located modular root ${modularRoot}`);
    return modularRoot;
  } catch (err) {
    throw new Error(err);
  }
}

var _default = (0, _memoize.default)(getModularRoot);

exports.default = _default;